searchData={"items":[{"type":"module","title":"Phoenix.Sync","doc":"Wrappers to ease integration of [Electric’s Postgres syncing\nservice](https://electric-sql.com) with [Phoenix\napplications](https://www.phoenixframework.org/).\n\nThere are currently 2 integration modes: [`Phoenix.LiveView`\nstreams](#module-phoenix-liveview-streams) and [configuration\ngateway](#module-configuration-gateway).","ref":"Phoenix.Sync.html"},{"type":"module","title":"Phoenix.LiveView Streams - Phoenix.Sync","doc":"`Phoenix.Sync.LiveView.sync_stream/4` integrates with\n[`Phoenix.LiveView.stream/4`](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#stream/4)\nand provides a live updating collection of items.","ref":"Phoenix.Sync.html#module-phoenix-liveview-streams"},{"type":"module","title":"Configuration Gateway - Phoenix.Sync","doc":"Using `Phoenix.Sync.Plug` you can create endpoints that\nreturn configuration information for your Electric Typescript clients. See\n[that module's documentation](`Phoenix.Sync.Plug`) for\nmore information.","ref":"Phoenix.Sync.html#module-configuration-gateway"},{"type":"module","title":"Installation - Phoenix.Sync","doc":"Add `phoenix_sync` to your application dependencies:\n\n    def deps do\n      [\n        {:phoenix_sync, \"~> 0.1\"}\n      ]\n    end","ref":"Phoenix.Sync.html#module-installation"},{"type":"module","title":"Configuration - Phoenix.Sync","doc":"In your `config/config.exs` or `config/runtime.exs` you **must** configure the\nclient for the Electric streaming API:\n\n    import Config\n\n    config :phoenix_sync, Electric.Client,\n      # one of `base_url` or `endpoint` is required\n      base_url: System.get_env(\"ELECTRIC_URL\", \"http://localhost:3000\"),\n      # endpoint: System.get_env(\"ELECTRIC_ENDPOINT\", \"http://localhost:3000/v1/shape\"),\n      # optional\n      database_id: System.get_env(\"ELECTRIC_DATABASE_ID\", nil)\n\nSee the documentation for [`Electric.Client.new/1`](`Electric.Client.new/1`)\nfor information on the client configuration.","ref":"Phoenix.Sync.html#module-configuration"},{"type":"module","title":"Embedding Electric - Phoenix.Sync","doc":"**TODO**","ref":"Phoenix.Sync.html#module-embedding-electric"},{"type":"function","title":"Phoenix.Sync.client!/0","doc":"Create a new `Electric.Client` instance based on the application config.\n\nTo connect your live streams to an externally hosted Electric instance over\nHTTP, configure your app with the URL of the Electric server:\n\n  # dev.exs\n  config :phoenix_sync, Electric.Client,\n    base_url: \"http://localhost:3000\"\n\nIf Electric is installed as a dependency of your app, and you wish to connect\nyour live streams to this internal application, then you don't need to configure\nanything -- `client!/0` will return an `Electric.Client` instance configured to\ndata directly from the running Electric application.\n\nSee [`Electric.Client.new/1`](`Electric.Client.new/1`) for the available\noptions.","ref":"Phoenix.Sync.html#client!/0"},{"type":"type","title":"Phoenix.Sync.param_override/0","doc":"","ref":"Phoenix.Sync.html#t:param_override/0"},{"type":"type","title":"Phoenix.Sync.param_overrides/0","doc":"","ref":"Phoenix.Sync.html#t:param_overrides/0"},{"type":"function","title":"Phoenix.Sync.plug_opts/0","doc":"","ref":"Phoenix.Sync.html#plug_opts/0"},{"type":"type","title":"Phoenix.Sync.shape_definition/0","doc":"","ref":"Phoenix.Sync.html#t:shape_definition/0"},{"type":"function","title":"Phoenix.Sync.shape_from_params/2","doc":"Use request query parameters to create a `Electric.Client.ShapeDefinition`.\n\nUseful when creating authorization endpoints that validate a user's access to\na specific shape.","ref":"Phoenix.Sync.html#shape_from_params/2"},{"type":"function","title":"Parameters - Phoenix.Sync.shape_from_params/2","doc":"","ref":"Phoenix.Sync.html#shape_from_params/2-parameters"},{"type":"function","title":"Required - Phoenix.Sync.shape_from_params/2","doc":"- `table` - the Postgres [table name](https://electric-sql.com/docs/guides/shapes#table)\n\n  Note: `table` is not required in the parameters if a `:table` override is set.","ref":"Phoenix.Sync.html#shape_from_params/2-required"},{"type":"function","title":"Optional - Phoenix.Sync.shape_from_params/2","doc":"- `where` - the [Shape's where clause](https://electric-sql.com/docs/guides/shapes#where-clause)\n- `columns` - The columns to include in the shape.\n- `namespace` - The Postgres namespace (also called `SCHEMA`).\n\nSee\n[`Electric.Client.ShapeDefinition.new/2`](`Electric.Client.ShapeDefinition.new/2`)\nfor more details on the parameters.","ref":"Phoenix.Sync.html#shape_from_params/2-optional"},{"type":"function","title":"Examples - Phoenix.Sync.shape_from_params/2","doc":"# pass the Plug.Conn struct for a request\n    iex> Phoenix.Sync.shape_from_params(%Plug.Conn{params: %{\"table\" => \"items\", \"where\" => \"visible = true\" }})\n    {:ok, %Electric.Client.ShapeDefinition{table: \"items\", where: \"visible = true\"}}\n\n    # or a simple parameter map\n    iex> Phoenix.Sync.shape_from_params(%{\"table\" => \"items\", \"columns\" => \"id,name,value\" })\n    {:ok, %Electric.Client.ShapeDefinition{table: \"items\", columns: [\"id\", \"name\", \"value\"]}}\n\n    iex> Phoenix.Sync.shape_from_params(%{\"columns\" => \"id,name,value\" })\n    {:error, \"Missing `table` parameter\"}","ref":"Phoenix.Sync.html#shape_from_params/2-examples"},{"type":"function","title":"Overriding Parameter Values - Phoenix.Sync.shape_from_params/2","doc":"If you want to hard-code some elements of the shape, ignoring the values from\nthe request, or to set defaults, then use the `overrides` to set specific\nvalues for elements of the shape.","ref":"Phoenix.Sync.html#shape_from_params/2-overriding-parameter-values"},{"type":"function","title":"Examples - Phoenix.Sync.shape_from_params/2","doc":"iex> Phoenix.Sync.shape_from_params(%{\"columns\" => \"id,name,value\"}, table: \"things\")\n    {:ok, %Electric.Client.ShapeDefinition{table: \"things\", columns: [\"id\", \"name\", \"value\"]}}\n\n    iex> Phoenix.Sync.shape_from_params(%{\"table\" => \"ignored\"}, table: \"things\")\n    {:ok, %Electric.Client.ShapeDefinition{table: \"things\"}}","ref":"Phoenix.Sync.html#shape_from_params/2-examples"},{"type":"type","title":"Phoenix.Sync.shape_specification/0","doc":"","ref":"Phoenix.Sync.html#t:shape_specification/0"},{"type":"module","title":"Phoenix.Sync.Application","doc":"","ref":"Phoenix.Sync.Application.html"},{"type":"function","title":"Phoenix.Sync.Application.plug_opts/0","doc":"Returns the required adapter configuration for your Phoenix Endpoint or\n`Plug.Router`.","ref":"Phoenix.Sync.Application.html#plug_opts/0"},{"type":"function","title":"Phoenix - Phoenix.Sync.Application.plug_opts/0","doc":"Configure your endpoint with the configuration at runtime by passing the\n`phoenix_sync` configuration to your endpoint in the `Application.start/2`\ncallback:\n\n    def start(_type, _args) do\n      children = [\n        # ...\n        {MyAppWeb.Endpoint, phoenix_sync: Phoenix.Sync.plug_opts()}\n      ]\n    end","ref":"Phoenix.Sync.Application.html#plug_opts/0-phoenix"},{"type":"function","title":"Plug - Phoenix.Sync.Application.plug_opts/0","doc":"Add the configuration to the Plug opts in your server configuration:\n\n    children = [\n      {Bandit, plug: {MyApp.Router, phoenix_sync: Phoenix.Sync.plug_opts()}}\n    ]\n\nYour `Plug.Router` must be configured with\n[`copy_opts_to_assign`](https://hexdocs.pm/plug/Plug.Builder.html#module-options) and you should `use` the rele\n\n    defmodule MyApp.Router do\n      use Plug.Router, copy_opts_to_assign: :options\n\n      use Phoenix.Sync.Controller\n      use Phoenix.Sync.Router\n\n      plug :match\n      plug :dispatch\n\n      sync \"/shapes/todos\", Todos.Todo\n\n      get \"/shapes/user-todos\" do\n        %{\"user_id\" => user_id} = conn.params\n        sync_render(conn, from(t in Todos.Todo, where: t.owner_id == ^user_id)\n      end\n    end","ref":"Phoenix.Sync.Application.html#plug_opts/0-plug"},{"type":"module","title":"Phoenix.Sync.Client","doc":"","ref":"Phoenix.Sync.Client.html"},{"type":"function","title":"Phoenix.Sync.Client.new/0","doc":"Create a new sync client based on the `:phoenix_sync` configuration.","ref":"Phoenix.Sync.Client.html#new/0"},{"type":"function","title":"Phoenix.Sync.Client.new/1","doc":"Create a sync client using the given options.\n\nIf the integration mode is set to `:embedded` and Electric is installed\nthen this will configure the client to retrieve data using the internal\nElixir APIs.\n\nFor the `:http` mode, then you must also configure a URL specifying an\nElectric API server:\n\n    config :phoenix_sync,\n      mode: :http,\n      url: \"https://api.electric-sql.cloud\"\n\nThis client can then generate streams for use in your Elixir applications:\n\n    client = Phoenix.Sync.Client.new()\n    stream = Electric.Client.stream(client, Todos.Todo)\n    for msg <- stream, do: IO.inspect(msg)\n\nAlternatively use `stream/1` which wraps this functionality.","ref":"Phoenix.Sync.Client.html#new/1"},{"type":"function","title":"Phoenix.Sync.Client.new!/0","doc":"","ref":"Phoenix.Sync.Client.html#new!/0"},{"type":"function","title":"Phoenix.Sync.Client.new!/1","doc":"","ref":"Phoenix.Sync.Client.html#new!/1"},{"type":"function","title":"Phoenix.Sync.Client.stream/2","doc":"Return a sync stream for the given shape.","ref":"Phoenix.Sync.Client.html#stream/2"},{"type":"function","title":"Examples - Phoenix.Sync.Client.stream/2","doc":"# stream updates for the Todo schema\n    stream = Phoenix.Sync.Client.stream(MyApp.Todos.Todo)\n\n    # stream the results of an ecto query\n    stream = Phoenix.Sync.Client.stream(from(t in MyApp.Todos.Todo, where: t.completed == true))\n\n    # create a stream based on a shape definition\n    stream = Phoenix.Sync.Client.stream(\n      table: \"todos\",\n      where: \"completed = false\",\n      columns: [\"id\", \"title\"]\n    )\n\n    # once you have a stream, consume it as usual\n    Enum.each(stream, &IO.inspect/1)","ref":"Phoenix.Sync.Client.html#stream/2-examples"},{"type":"function","title":"Ecto vs keyword shapes - Phoenix.Sync.Client.stream/2","doc":"Streams defined using an Ecto query or schema will return data wrapped in\nthe appropriate schema struct, with values cast to the appropriate\nElixir/Ecto types, rather than raw column data in the form `%{\"column_name\"\n=> \"column_value\"}`.","ref":"Phoenix.Sync.Client.html#stream/2-ecto-vs-keyword-shapes"},{"type":"function","title":"Phoenix.Sync.Client.stream/3","doc":"","ref":"Phoenix.Sync.Client.html#stream/3"},{"type":"module","title":"Phoenix.Sync.Controller","doc":"Provides controller-level integration with sync streams.\n\nUnlike `Phoenix.Sync.Router.sync/2`, which only permits static shape\ndefinitions, in a controller you can use request and session information to\nfilter your data.","ref":"Phoenix.Sync.Controller.html"},{"type":"module","title":"Phoenix Example - Phoenix.Sync.Controller","doc":"defmodule MyAppWeb.TodoController do\n      use Phoenix.Controller, formats: [:html, :json]\n\n      import Elixir.Phoenix.Sync.Controller\n\n      alias MyApp.Todos\n\n      def all(conn, %{\"user_id\" => user_id} = params) do\n        sync_render(\n          conn,\n          params,\n          from(t in Todos.Todo, where: t.owner_id == ^user_id)\n        )\n      end\n    end","ref":"Phoenix.Sync.Controller.html#module-phoenix-example"},{"type":"module","title":"Plug Example - Phoenix.Sync.Controller","doc":"You should `use Elixir.Phoenix.Sync.Controller` in your `Plug.Router`, then within your route\nyou can use the `sync_render/2` function.\n\n    defmodule MyPlugApp.Router do\n      use Plug.Router, copy_opts_to_assign: :options\n      use Elixir.Phoenix.Sync.Controller\n\n      plug :match\n      plug :dispatch\n\n      get \"/todos\" do\n        sync_render(conn, MyPlugApp.Todos.Todo)\n      end\n    end","ref":"Phoenix.Sync.Controller.html#module-plug-example"},{"type":"function","title":"Phoenix.Sync.Controller.sync_render/3","doc":"Return the sync events for the given shape as a `Plug.Conn` response.","ref":"Phoenix.Sync.Controller.html#sync_render/3"},{"type":"module","title":"Phoenix.Sync.Electric","doc":"A `Plug.Router` and `Phoenix.Router` compatible Plug handler that allows\nyou to mount the full Electric shape api into your application.\n\nUnlike `Phoenix.Sync.Router.sync/2` this allows your app to serve\nshapes defined by `table` parameters, much like the Electric application.\n\nThe advantage is that you're free to put your own authentication and\nauthorization Plugs in front of this endpoint, integrating the auth for\nyour shapes API with the rest of your app.","ref":"Phoenix.Sync.Electric.html"},{"type":"module","title":"Configuration - Phoenix.Sync.Electric","doc":"Before configuring your router, you must install and configure the\n`:phoenix_sync` application.\n\nSee the documentation for [embedding electric](`Phoenix.Sync`) for\ndetails on embedding Electric into your Elixir application.","ref":"Phoenix.Sync.Electric.html#module-configuration"},{"type":"module","title":"Plug Integration - Phoenix.Sync.Electric","doc":"Mount this Plug into your router using `Plug.Router.forward/2`.\n\n    defmodule MyRouter do\n      use Plug.Router, copy_opts_to_assign: :config\n      use Phoenix.Sync.Electric\n\n      plug :match\n      plug :dispatch\n\n      forward \"/shapes\",\n        to: Phoenix.Sync.Electric,\n        init_opts: [opts_in_assign: :config]\n    end\n\nYou **must** configure your `Plug.Router` with `copy_opts_to_assign` and\npass the key you configure here (in this case `:config`) to the\n`Phoenix.Sync.Electric` plug in it's `init_opts`.\n\nIn your application, build your Electric confguration using\n`Phoenix.Sync.plug_opts()` and pass the result to your router as\n`phoenix_sync`:\n\n    # in application.ex\n    def start(_type, _args) do\n      children = [\n        {Bandit, plug: {MyRouter, phoenix_sync: Phoenix.Sync.plug_opts()}, port: 4000}\n      ]\n\n      Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n    end","ref":"Phoenix.Sync.Electric.html#module-plug-integration"},{"type":"module","title":"Phoenix Integration - Phoenix.Sync.Electric","doc":"Use `Phoenix.Router.forward/2` in your router:\n\n    defmodule MyAppWeb.Router do\n      use Phoenix.Router\n\n      pipeline :shapes do\n        # your authz plugs\n      end\n\n      scope \"/shapes\" do\n        pipe_through [:shapes]\n\n        forward \"/\", Phoenix.Sync.Electric\n      end\n    end\n\nAs for the Plug integration, include the configuration at runtime\nwithin the `Application.start/2` callback.\n\n    # in application.ex\n    def start(_type, _args) do\n      children = [\n        # ...\n        {MyAppWeb.Endpoint, phoenix_sync: Phoenix.Sync.plug_opts()}\n      ]\n\n      Supervisor.start_link(children, strategy: :one_for_one, name: MyApp.Supervisor)\n    end","ref":"Phoenix.Sync.Electric.html#module-phoenix-integration"},{"type":"module","title":"Phoenix.Sync.LiveView","doc":"","ref":"Phoenix.Sync.LiveView.html"},{"type":"opaque","title":"Phoenix.Sync.LiveView.component_event/0","doc":"","ref":"Phoenix.Sync.LiveView.html#t:component_event/0"},{"type":"function","title":"Phoenix.Sync.LiveView.electric_client_configuration/1","doc":"Embed client configuration for a shape into your HTML.\n\n     \n\nThis will put a ` ` tag into your page setting\n`window.todo_shape_config` to the configuration needed to subscribe to\nchanges to the `MyApp.Todo` table.\n\n     \n      window.todo_shape_config = {\"url\":\"https://localhost:3000/v1/shape/todos\" /* , ... */}\n     \n\nIf you include a `:script` slot then you have complete control over how the\nconfiguration is applied.\n\n     \n      <:script :let={configuration}>\n        const container = document.getElementById(\"root\")\n        const root = createRoot(container)\n        root.render(\n          React.createElement(\n            MyApp, {\n              client_config: <%= configuration %>\n            },\n            null\n          )\n        );\n      </:script>\n     \n\nThe `configuration` variable in the `:script` block is the  JSON-encoded\nclient configuration.\n\n     \n      const container = document.getElementById(\"root\")\n      const root = createRoot(container)\n      root.render(\n        React.createElement(\n          MyApp, {\n            client_config: {\"url\":\"https://localhost:3000/v1/shape/todos\" /* , ... */}\n          },\n          null\n        )\n      );","ref":"Phoenix.Sync.LiveView.html#electric_client_configuration/1"},{"type":"function","title":"Attributes - Phoenix.Sync.LiveView.electric_client_configuration/1","doc":"* `shape` (`:any`) (required) - The Ecto query (or schema module) to subscribe to.\n* `key` (`:string`) - The key in the top-level `window` object to put the configuration object. Defaults to `\"electric_client_config\"`.\n* `client` (`:any`) - Optional client. If not set defaults to `Phoenix.Sync.client!()`.","ref":"Phoenix.Sync.LiveView.html#electric_client_configuration/1-attributes"},{"type":"function","title":"Slots - Phoenix.Sync.LiveView.electric_client_configuration/1","doc":"* `script` - An optional inner block that allows you to override what you want to do with the configuration JSON.","ref":"Phoenix.Sync.LiveView.html#electric_client_configuration/1-slots"},{"type":"type","title":"Phoenix.Sync.LiveView.event/0","doc":"","ref":"Phoenix.Sync.LiveView.html#t:event/0"},{"type":"opaque","title":"Phoenix.Sync.LiveView.replication_event/0","doc":"","ref":"Phoenix.Sync.LiveView.html#t:replication_event/0"},{"type":"opaque","title":"Phoenix.Sync.LiveView.root_event/0","doc":"","ref":"Phoenix.Sync.LiveView.html#t:root_event/0"},{"type":"type","title":"Phoenix.Sync.LiveView.state_event/0","doc":"","ref":"Phoenix.Sync.LiveView.html#t:state_event/0"},{"type":"type","title":"Phoenix.Sync.LiveView.stream_option/0","doc":"","ref":"Phoenix.Sync.LiveView.html#t:stream_option/0"},{"type":"type","title":"Phoenix.Sync.LiveView.stream_options/0","doc":"","ref":"Phoenix.Sync.LiveView.html#t:stream_options/0"},{"type":"function","title":"Phoenix.Sync.LiveView.sync_stream/4","doc":"Maintains a LiveView stream from the given Ecto query.\n\n- `name` The name to use for the LiveView stream.\n- `query` An [`Ecto`](`Ecto`) query that represents the data to stream from the database.\n\nFor example:\n\n    def mount(_params, _session, socket) do\n      socket =\n        Phoenix.Sync.LiveView.sync_stream(\n          socket,\n          :admins,\n          from(u in Users, where: u.admin == true)\n        )\n      {:ok, socket}\n    end\n\nThis will subscribe to the configured Electric server and keep the list of\n`:admins` in sync with the database via a `Phoenix.LiveView` stream.\n\nUpdates will be delivered to the view via messages to the LiveView process.\n\nTo handle these you need to add a `handle_info/2` implementation that receives these:\n\n    def handle_info({:sync, event}, socket) do\n      {:noreply, Phoenix.Sync.LiveView.sync_stream_update(socket, event)}\n    end\n\nSee the docs for\n[`Phoenix.LiveView.stream/4`](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html#stream/4)\nfor details on using LiveView streams.","ref":"Phoenix.Sync.LiveView.html#sync_stream/4"},{"type":"function","title":"Lifecycle Events - Phoenix.Sync.LiveView.sync_stream/4","doc":"Most `{:sync, event}` messages are opaque and should be passed directly\nto the `sync_stream_update/3` function, but there are two events that are\noutside Electric's replication protocol and designed to be useful in the\nLiveView component.\n\n- `{:sync, {stream_name, :loaded}}` - sent when the Electric event stream has passed\n  from initial state to update mode.\n\n  This event is useful to show the stream component after\n  the initial sync. Because of the streaming nature of Electric Shapes, the\n  intitial sync can cause flickering as items are added, removed and updated.\n\n  E.g.:\n\n      # in the LiveView component\n      def handle_info({:sync, {_name, :live}}, socket) do\n        {:noreply, assign(socket, :show_stream, true)}\n      end\n\n      # in the template\n       \n         \n          <%= item.value %>\n         \n       \n\n- `{:sync, {stream_name, :live}}` - sent when the Electric stream is in\n  `live` mode, that is the initial state has loaded and the client is\n  up-to-date with the database and is long-polling for new events from the\n  Electric server.\n\nIf your app doesn't need this extra information, then you can ignore them and\njust have a catch-all callback:\n\n    def handle_info({:sync, event}, socket) do\n      {:noreply, Phoenix.Sync.LiveView.sync_stream_update(socket, event)}\n    end\n\n`Phoenix.Sync.LiveView.sync_stream_update` will just ignore the\nlifecycle events.","ref":"Phoenix.Sync.LiveView.html#sync_stream/4-lifecycle-events"},{"type":"function","title":"Sub-components - Phoenix.Sync.LiveView.sync_stream/4","doc":"If you register your Electric stream in a sub-component you will still\nreceive Electric messages in the LiveView's root/parent process.\n\n`Phoenix.Sync` handles this for you by encapsulating component messages\nso it can correctly forward on the event to the component.\n\nSo in the parent `LiveView` process you handle the `:sync` messages as\nabove:\n\n    defmodule MyLiveView do\n      use Phoenix.LiveView\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n          <.live_component id=\"my_component\" module={MyComponent} />\n         \n        \"\"\"\n      end\n\n      # We setup the Electric sync_stream in the component but update messages will\n      # be sent to the parent process.\n      def handle_info({:sync, event}, socket) do\n        {:noreply, Phoenix.Sync.LiveView.sync_stream_update(socket, event)}\n      end\n    end\n\nIn the component you must handle these events in the\n`c:Phoenix.LiveComponent.update/2` callback:\n\n    defmodule MyComponent do\n      use Phoenix.LiveComponent\n\n      def render(assigns) do\n        ~H\"\"\"\n         \n           \n            <%= user.name %>\n           \n         \n        \"\"\"\n      end\n\n      # Equivalent to the `handle_info({:sync, {stream_name, :live}}, socket)` callback\n      # in the parent LiveView.\n      def update(%{sync: {_stream_name, :live}}, socket) do\n        {:ok, socket}\n      end\n\n      # Equivalent to the `handle_info({:sync, event}, socket)` callback\n      # in the parent LiveView.\n      def update(%{sync: event}, socket) do\n        {:ok, Phoenix.Sync.LiveView.sync_stream_update(socket, event)}\n      end\n\n      def update(assigns, socket) do\n        {:ok, Phoenix.Sync.LiveView.sync_stream(socket, :users, User)}\n      end\n    end","ref":"Phoenix.Sync.LiveView.html#sync_stream/4-sub-components"},{"type":"function","title":"Phoenix.Sync.LiveView.sync_stream_update/3","doc":"Handle Electric events within a LiveView.\n\n    def handle_info({:sync, event}, socket) do\n      {:noreply, Phoenix.Sync.LiveView.sync_stream_update(socket, event, at: 0)}\n    end\n\nThe `opts` are passed to the `Phoenix.LiveView.stream_insert/4` call.","ref":"Phoenix.Sync.LiveView.html#sync_stream_update/3"},{"type":"module","title":"Phoenix.Sync.Plug","doc":"Provides an configuration endpoint for use in your Phoenix applications.\n\nRather than configuring your [Electric Typescript\nclient](https://electric-sql.com/docs/api/clients/typescript) directly, you\ninstead configure a route in your application with a pre-configured\n`Electric.Client.ShapeDefinition` and then retreive the URL and other\nconfiguration for that shape from your client via a request to your Phoenix\napplication.\n\nIn your Phoenix application, [add a route](https://hexdocs.pm/phoenix/Phoenix.Router.html) to\n`Phoenix.Sync.Plug` specifying a particular shape:\n\n    defmodule MyAppWeb.Router do\n      scope \"/shapes\" do\n        pipe_through :browser\n\n        get \"/todos\", Phoenix.Sync.Plug,\n          shape: Electric.Client.shape!(\"todos\", where: \"visible = true\")\n      end\n    end\n\nThen in your client code, you retrieve the shape configuration directly\nfrom the Phoenix endpoint:\n\n``` typescript\nimport { ShapeStream } from '@electric-sql/client'\n\nconst endpoint = `https://localhost:4000/shapes/todos`\nconst response = await fetch(endpoint)\nconst config = await response.json()\n\n// The returned `config` has all the information you need to subscribe to\n// your shape\nconst stream = new ShapeStream(config)\n\nstream.subscribe(messages => {\n  // ...\n})\n```\n\nYou can add additional authentication/authorization for shapes using\n[Phoenix's\npipelines](https://hexdocs.pm/phoenix/Phoenix.Router.html#pipeline/2) or\nother [`plug`\ncalls](https://hexdocs.pm/phoenix/Phoenix.Router.html#plug/2).","ref":"Phoenix.Sync.Plug.html"},{"type":"module","title":"Plug.Router - Phoenix.Sync.Plug","doc":"For  pure `Plug`-based applications, you can use `Plug.Router.forward/2`:\n\n    defmodule MyRouter do\n      use Plug.Router\n\n      plug :match\n      plug :dispatch\n\n      forward \"/shapes/items\",\n        to: Phoenix.Sync.Plug,\n        shape: Electric.Client.shape!(\"items\")\n\n      match _ do\n        send_resp(conn, 404, \"oops\")\n      end\n    end","ref":"Phoenix.Sync.Plug.html#module-plug-router"},{"type":"module","title":"Parameter-based shapes - Phoenix.Sync.Plug","doc":"As well as defining fixed-shapes for a particular url, you can request\nshape configuration using parameters in your request:\n\n    defmodule MyAppWeb.Router do\n      scope \"/\" do\n        pipe_through :browser\n\n        get \"/shape\", Phoenix.Sync.Plug, []\n      end\n    end\n\n``` typescript\nimport { ShapeStream } from '@electric-sql/client'\n\nconst endpoint = `https://localhost:4000/shape?table=items&namespace=public&where=visible%20%3D%20true`\nconst response = await fetch(endpoint)\nconst config = await response.json()\n\n// as before\n```\n\nThe parameters are:\n\n- `table` - The Postgres table name (required).\n- `namespace` - The Postgres schema if not specified defaults to `public`.\n- `where` - The [where clause](https://electric-sql.com/docs/guides/shapes#where-clause) to filter items from the shape.","ref":"Phoenix.Sync.Plug.html#module-parameter-based-shapes"},{"type":"module","title":"Custom Authentication - Phoenix.Sync.Plug","doc":"The `Electric.Client` allows for generating authentication headers via it's\n`authenticator` configuration but if you want to include parameters from the\nrequest in the authentication tokens, for example including the currently\nlogged in user id, then you can set an `authenticator` directly in the `Phoenix.Sync.Plug` configuration.\n\nFirst you must define a function that accepts the `%Plug.Conn{}` of the\nrequest, the `%Electric.Client.ShapeDefinition{}` of the endpoint and some\n(optional) config and returns a map of additional request headers:\n\n    defmodule MyAuthModule do\n      def shape_auth_headers(conn, shape, _opts \\\\ []) do\n        user_id = conn.assigns.user_id\n        signer = Joken.Signer.create(\"HS256\", \"my-deep-secret\")\n        claims = %{\n          user_id: user_id,\n          table: [shape.namespace, shape.table],\n          where: shape.where\n        }\n        token = Joken.generate_and_sign!(Joken.Config.default_claims(), claims, signer)\n        %{\"authorization\" => \"Bearer #{token}\"}\n      end\n    end\n\nNow configure the Shape configuration endpoint to use your authentication function:\n\n    forward \"/shapes/tasks/:project_id\",\n      to: Electric.Plug,\n      authenticator: {MyAuthModule, :shape_auth_headers, _opts = []},\n      shape: [\n        from(t in Task, where: t.active == true),\n        project_id: :project_id\n      ]\n\n\nOr you can use a capture:\n\n    forward \"/shapes/tasks/:project_id\",\n      to: Electric.Plug,\n      authenticator: &MyAuthModule.shape_auth_headers/2,\n      shape: [\n        from(t in Task, where: t.active == true),\n        project_id: :project_id\n      ]","ref":"Phoenix.Sync.Plug.html#module-custom-authentication"},{"type":"function","title":"Phoenix.Sync.Plug.call/2","doc":"","ref":"Phoenix.Sync.Plug.html#call/2"},{"type":"type","title":"Phoenix.Sync.Plug.conn_param_spec/0","doc":"","ref":"Phoenix.Sync.Plug.html#t:conn_param_spec/0"},{"type":"type","title":"Phoenix.Sync.Plug.dynamic_shape_param/0","doc":"","ref":"Phoenix.Sync.Plug.html#t:dynamic_shape_param/0"},{"type":"type","title":"Phoenix.Sync.Plug.dynamic_shape_params/0","doc":"","ref":"Phoenix.Sync.Plug.html#t:dynamic_shape_params/0"},{"type":"type","title":"Phoenix.Sync.Plug.op/0","doc":"","ref":"Phoenix.Sync.Plug.html#t:op/0"},{"type":"type","title":"Phoenix.Sync.Plug.param_name/0","doc":"","ref":"Phoenix.Sync.Plug.html#t:param_name/0"},{"type":"function","title":"Phoenix.Sync.Plug.send_configuration/3","doc":"Send the client configuration for a given shape to the browser.","ref":"Phoenix.Sync.Plug.html#send_configuration/3"},{"type":"function","title":"Example - Phoenix.Sync.Plug.send_configuration/3","doc":"get \"/my-shapes/messages\" do\n      user_id = get_session(conn, :user_id)\n      shape = from(m in Message, where: m.user_id == ^user_id)\n      Phoenix.Sync.Plug.send_configuration(conn, shape)\n    end\n\n    get \"/my-shapes/tasks/:project_id\" do\n      project_id = conn.params[\"project_id\"]\n\n      if user_has_access_to_project?(project_id) do\n        shape = where(Task, project_id: ^project_id)\n        Phoenix.Sync.Plug.send_configuration(conn, shape)\n      else\n        send_resp(conn, :forbidden, \"You do not have permission to view project #{project_id}\")\n      end\n    end","ref":"Phoenix.Sync.Plug.html#send_configuration/3-example"},{"type":"function","title":"Phoenix.Sync.Plug.shape!/2","doc":"Defines a shape based on a root `Ecto` query plus some filters based on the\ncurrent request.\n\n    forward \"/shapes/tasks/:project_id\",\n      to: Electric.Plug,\n      shape: Phoenix.Sync.Plug.shape!(\n        from(t in Task, where: t.active == true),\n        project_id: :project_id\n      )\n\nThe `params` describe the way to build the `where` clause on the shape from\nthe request parameters.\n\nFor example, `[id: :user_id]` means that the `id` column on the table should\nmatch the value of the `user_id` parameter, equivalent to:\n\n    from(\n      t in Table,\n      where: t.id == ^conn.params[\"user_id\"]\n    )\n\nIf both the table column and the request parameter have the same name, then\nyou can just pass the name, so:\n\n    Phoenix.Sync.Plug.shape!(Table, [:visible])\n\nis equivalent to:\n\n    from(\n      t in Table,\n      where: t.visible == ^conn.params[\"visible\"]\n    )\n\nIf you need to match on something other than `==` then you can pass the operator in the params:\n\n    Phoenix.Sync.Plug.shape!(Table, size: [>=: :size])\n\nis equivalent to:\n\n    from(\n      t in Table,\n      where: t.size >= ^conn.params[\"size\"]\n    )\n\nInstead of calling `shape!/2` directly in your route definition, you can just\npass a list of `[query | params]` to do the same thing:\n\n    forward \"/shapes/tasks/:project_id\",\n      to: Electric.Plug,\n      shape: [\n        from(t in Task, where: t.active == true),\n        project_id: :project_id\n      ]","ref":"Phoenix.Sync.Plug.html#shape!/2"},{"type":"type","title":"Phoenix.Sync.Plug.table_column/0","doc":"","ref":"Phoenix.Sync.Plug.html#t:table_column/0"},{"type":"module","title":"Phoenix.Sync.Router","doc":"Provides router macros to simplify the exposing of Electric shape streams\nwithin your Phoenix or Plug application.","ref":"Phoenix.Sync.Router.html"},{"type":"module","title":"Phoenix Integration - Phoenix.Sync.Router","doc":"When using within a Phoenix application, you should just import the macros\ndefined here\n\n    import Elixir.Phoenix.Sync.Router","ref":"Phoenix.Sync.Router.html#module-phoenix-integration"},{"type":"module","title":"Plug Integration - Phoenix.Sync.Router","doc":"Within Plug applications you need to do a little more work.\n\nTODO","ref":"Phoenix.Sync.Router.html#module-plug-integration"},{"type":"macro","title":"Phoenix.Sync.Router.sync/2","doc":"Defines a synchronization route for streaming Electric shapes.\n\nThe shape can be defined in several ways:","ref":"Phoenix.Sync.Router.html#sync/2"},{"type":"macro","title":"Using Ecto Schemas - Phoenix.Sync.Router.sync/2","doc":"Defines a synchronization route for streaming Electric shapes using an Ecto schema.\n\n    sync \"/all-todos\", MyApp.Todo\n\nNote: Only Ecto schema modules are supported as direct arguments. For Ecto queries,\nuse the `query` option in the third argument or use `Phoenix.Sync.Controller.sync_render/3`.","ref":"Phoenix.Sync.Router.html#sync/2-using-ecto-schemas"},{"type":"macro","title":"Using Ecto Schema and `where` clause - Phoenix.Sync.Router.sync/2","doc":"sync \"/incomplete-todos\", MyApp.Todo, where: \"completed = false\"","ref":"Phoenix.Sync.Router.html#sync/2-using-ecto-schema-and-where-clause"},{"type":"macro","title":"Using an explicit `table` - Phoenix.Sync.Router.sync/2","doc":"sync \"/incomplete-todos\", table: \"todos\", where: \"completed = false\"","ref":"Phoenix.Sync.Router.html#sync/2-using-an-explicit-table"},{"type":"macro","title":"Options - Phoenix.Sync.Router.sync/2","doc":"* `:table` - (required if no schema provided) The database table to expose\n  * `:namespace` - (optional) The namespace for the table\n  * `:where` - (optional) The where clause to apply to the data\n  * `:columns` - (optional) Subset of table columns to include in the streamed data (default all columns)\n  * `:storage` - (optional) Storage configuration\n  * `:replica` - (optional) Replication strategy","ref":"Phoenix.Sync.Router.html#sync/2-options"},{"type":"macro","title":"Phoenix.Sync.Router.sync/3","doc":"","ref":"Phoenix.Sync.Router.html#sync/3"},{"type":"extras","title":"README","doc":"# Phoenix.Sync\n\n[![Hex.pm](https://img.shields.io/hexpm/v/phoenix_sync.svg)](https://hex.pm/packages/phoenix_sync)\n[![Documentation](https://img.shields.io/badge/docs-hexdocs-green)](https://hexdocs.pm/phoenix_sync)\n[![License](https://img.shields.io/badge/license-Apache_2.0-green)](./LICENSE)\n[![Status](https://img.shields.io/badge/status-beta-orange)](https://github.com/electric-sql/phoenix_sync)\n[![Discord](https://img.shields.io/discord/933657521581858818?color=5969EA&label=discord)](https://discord.electric-sql.com)\n\nSync is the best way of building modern apps. Phoenix.Sync enables real-time sync for Postgres-backed [Phoenix](https://www.phoenixframework.org/) applications.\n\nDocumentation is available at [hexdocs.pm/phoenix_sync](https://hexdocs.pm/phoenix_sync).","ref":"readme.html"},{"type":"extras","title":"Build real-time apps on locally synced data - README","doc":"- sync data into Elixir, `LiveView` and frontend web and mobile applications\n- integrates with `Plug` and `Phoenix.{Controller, LiveView, Router, Stream}`\n- uses [ElectricSQL](https://electric-sql.com) for scalable data delivery and fan out\n- maps `Ecto` queries to [Shapes](https://electric-sql.com/docs/guides/shapes) for partial replication","ref":"readme.html#build-real-time-apps-on-locally-synced-data"},{"type":"extras","title":"Usage - README","doc":"There are four key APIs:\n\n- [`Phoenix.Sync.Client.stream/2`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.Client.html#stream/2) for low level usage in Elixir\n- [`Phoenix.Sync.LiveView.sync_stream/4`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.LiveView.html#sync_stream/4) to sync into a LiveView stream\n- [`Phoenix.Sync.Router.sync/2`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.Router.html#sync/2) macro to expose a statically defined shape in your Router\n- [`Phoenix.Sync.Controller.sync_render/3`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.Controller.html#sync_render/3) to expose dynamically constructed shapes from a Controller","ref":"readme.html#usage"},{"type":"extras","title":"Low level usage in Elixir - README","doc":"Use [`Phoenix.Sync.Client.stream/2`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.Client.html#stream/2) to convert an `Ecto.Query` into an Elixir `Stream`:\n\n```elixir\nstream = Phoenix.Sync.Client.stream(Todos.Todo)\n\nstream =\n  Ecto.Query.from(t in Todos.Todo, where: t.completed == false)\n  |> Phoenix.Sync.Client.stream()\n```","ref":"readme.html#low-level-usage-in-elixir"},{"type":"extras","title":"Sync into a LiveView stream - README","doc":"Swap out `Phoenix.LiveView.stream/3` for [`Phoenix.Sync.LiveView.sync_stream/4`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.LiveView.html#sync_stream/4) to automatically keep a LiveView up-to-date with the state of your Postgres database:\n\n```elixir\ndefmodule MyWeb.MyLive do\n  use Phoenix.LiveView\n  import Phoenix.Sync.LiveView\n\n  def mount(_params, _session, socket) do\n    {:ok, sync_stream(socket, :todos, Todos.Todo)}\n  end\n\n  def handle_info({:sync, event}, socket) do\n    {:noreply, sync_stream_update(socket, event)}\n  end\nend\n```\n\nLiveView takes care of automatically keeping the front-end up-to-date with the assigned stream. What Phoenix.Sync does is automatically keep the _stream_ up-to-date with the state of the database.\n\nThis means you can build fully end-to-end real-time multi-user applications without writing Javascript _and_ without worrying about message delivery, reconnections, cache invalidation or polling the database for changes.","ref":"readme.html#sync-into-a-liveview-stream"},{"type":"extras","title":"Sync shapes through your Router - README","doc":"Use the [`Phoenix.Sync.Router.sync/2`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.Router.html#sync/2) macro to expose statically (compile-time) defined shapes in your Router:\n\n```elixir\ndefmodule MyWeb.Router do\n  use Phoenix.Router\n  import Phoenix.Sync.Router\n\n  pipeline :sync do\n    plug :my_auth\n  end\n\n  scope \"/shapes\" do\n    pipe_through :sync\n\n    sync \"/todos\", Todos.Todo\n  end\nend\n```\n\nBecause the shapes are exposed through your Router, the client connects through your existing Plug middleware. This allows you to do real-time sync straight out of Postgres _without_ having to translate your auth logic into complex/fragile database rules.","ref":"readme.html#sync-shapes-through-your-router"},{"type":"extras","title":"Sync dynamic shapes from a Controller - README","doc":"Sync shapes from any standard Controller using the [`Phoenix.Sync.Controller.sync_render/3`](https://hexdocs.pm/phoenix_sync/Phoenix.Sync.Controller.html#sync_render/3) view function:\n\n```elixir\ndefmodule Phoenix.Sync.LiveViewTest.TodoController do\n  use Phoenix.Controller\n  import Phoenix.Sync.Controller\n  import Ecto.Query, only: [from: 2]\n\n  def show(conn, %{\"done\" => done} = params) do\n    sync_render(conn, params, from(t in Todos.Todo, where: t.done == ^done))\n  end\n\n  def show_mine(%{assigns: %{current_user: user_id}} = conn, params) do\n    sync_render(conn, params, from(t in Todos.Todo, where: t.owner_id == ^user_id))\n  end\nend\n```\n\nThis allows you to define and personalise the shape definition at runtime using the session and request.","ref":"readme.html#sync-dynamic-shapes-from-a-controller"},{"type":"extras","title":"Consume shapes in the frontend - README","doc":"You can sync _into_ any client in any language that [speaks HTTP and JSON](https://electric-sql.com/docs/api/http).\n\nFor example, using the Electric [sync_stream_updatescript client](https://electric-sql.com/docs/api/clients/typescript):\n\n```sync_stream_updatescript\nimport { Shape, ShapeStream } from \"@electric-sql/client\";\n\nconst stream = new ShapeStream({\n  url: `/shapes/todos`,\n});\nconst shape = new Shape(stream);\n\n// The callback runs every time the data changes.\nshape.subscribe((data) => console.log(data));\n```\n\nOr binding a shape to a component using the [React bindings](https://electric-sql.com/docs/integrations/react):\n\n```tsx\nimport { useShape } from \"@electric-sql/react\";\n\nconst MyComponent = () => {\n  const { data } = useShape({\n    url: `shapes/todos`,\n  });\n\n  return  ;\n};\n```\n\nSee the Electric [demos](https://electric-sql.com/demos) and [documentation](https://electric-sql.com/demos) for more client-side usage examples.","ref":"readme.html#consume-shapes-in-the-frontend"},{"type":"extras","title":"Shape Definitions - README","doc":"Phoenix.Sync allows shapes to be defined in two ways:\n\n#### As an `Ecto` schema module or `Ecto.Query`\n\nThis is the simplest way to\nintegrate with your existing data model and means you don't have to worry too\nmuch about the lower-level details of the integration.\n\nExamples:\n\n```elixir\n\nsync_render(conn, params, from(t in Todos.Todo, where: t.completed == false))\n```\n\nQuery support is currently limited to only `where` conditions. Support for more complex queries, including `JOIN`s is planned.\n\n**Note:** The static shapes defined using the `sync/2` or `sync/3` router macros do not accept `Ecto.Query` structs as a shape definition. This is to avoid excessive recompilation caused by having your router having a compile-time dependency on your `Ecto` schemas.\n\nIf you want to add a where-clause filter to a static shape in your router, you must add an explicit [`where` clause](https://electric-sql.com/docs/guides/shapes#where-clause) alongside your `Ecto.Schema` module:\n\n```elixir\n\nsync \"/incomplete-todos\", Todos.Todo, where: \"completed = false\"\n```\n\nYou can also include `replica` (see below) in your static shape definitions:\n\n```elixir\n\nsync \"/incomplete-todos\", Todos.Todo, where: \"completed = false\", replica: :full\n```\n\n#### As a keyword list\n\nAt minimum a shape requires a `table`. You can think of shapes defined with\njust a table name as the sync-equivalent of `SELECT * FROM table`.\n\nThe available options are:\n\n- `table` (required). The Postgres table name. Be aware of casing and [Postgres's handling of unquoted upper-case names](https://wiki.postgresql.org/wiki/Don%27t_Do_This#Don.27t_use_upper_case_table_or_column_names).\n- `namespace` (optional). The Postgres namespace that the table belongs to. Defaults to `public`.\n- `where` (optional). Filter to apply to the synced data in SQL syntax, e.g. `where: \"amount < 1.23 AND colour in ('red', 'green')`.\n- `columns` (optional). The columns to include in the synced data. By default Electric will include all columns in the table. The column list **must** include all primary keys. E.g. `columns: [\"id\", \"title\", \"amount\"]`.\n- `replica` (optional). By default Electric will only send primary keys + changed columns on updates. Set `replica: :full` to receive the full row, not just the changed columns.","ref":"readme.html#shape-definitions"},{"type":"extras","title":"Installation and configuration - README","doc":"`Phoenix.Sync` can be used in two modes:\n\n1. `:embedded` where Electric is included as an application dependency and Phoenix.Sync consumes data internally using Elixir APIs\n2. `:http` where Electric does _not_ need to be included as an application dependency and Phoenix.Sync consumes data from an external Electric service using it's [HTTP API](https://electric-sql.com/docs/api/http)","ref":"readme.html#installation-and-configuration"},{"type":"extras","title":"Embedded mode - README","doc":"In `:embedded` mode, Electric must be included an application dependency but does not expose an HTTP API (internally or externally). Messages are streamed internally between Electric and Phoenix.Sync using Elixir function APIs. The only HTTP API for sync is that exposed via your Phoenix Router using the `sync/2` macro and `sync_render/3` function.\n\nExample config:\n\n```elixir\n# mix.exs\ndefp deps do\n  [\n    {:electric, \">= 1.0.0-beta.20\"},\n    {:phoenix_sync, \"~> 0.3\"}\n  ]\nend\n\n# config/config.exs\nconfig :phoenix_sync,\n  mode: :embedded,\n  env: config_env(),\n  repo: MyApp.Repo\n\n# application.ex\nchildren = [\n  MyApp.Repo,\n  # ...\n  {MyApp.Endpoint, phoenix_sync: Phoenix.Sync.plug_opts()}\n]\n```","ref":"readme.html#embedded-mode"},{"type":"extras","title":"HTTP - README","doc":"In `:http` mode, Electric does not need to be included as an application dependency. Instead, Phoenix.Sync consumes data from an external Electric service over HTTP.\n\n```elixir\n# mix.exs\ndefp deps do\n  [\n    {:phoenix_sync, \"~> 0.3\"}\n  ]\nend\n\n# config/config.exs\nconfig :phoenix_sync,\n  mode: :http,\n  env: config_env(),\n  url: \"https://api.electric-sql.cloud\",\n  credentials: [\n    secret: \"...\",    # required\n    source_id: \"...\"  # optional, required for Electric Cloud\n  ]\n\n# application.ex\nchildren = [\n  MyApp.Repo,\n  # ...\n  {MyApp.Endpoint, phoenix_sync: Phoenix.Sync.plug_opts()}\n]\n```","ref":"readme.html#http"},{"type":"extras","title":"Local HTTP services - README","doc":"It is also possible to include Electric as an application dependency and configure it to expose a local HTTP API that's consumed by Phoenix.Sync running in `:http` mode:\n\n```elixir\n# mix.exs\ndefp deps do\n  [\n    {:electric, \">= 1.0.0-beta.20\"},\n    {:phoenix_sync, \"~> 0.3\"}\n  ]\nend\n\n# config/config.exs\nconfig :phoenix_sync,\n  mode: :http,\n  env: config_env(),\n  http: [\n    port: 3000,\n  ],\n  repo: MyApp.Repo,\n  url: \"http://localhost:3000\"\n\n# application.ex\nchildren = [\n  MyApp.Repo,\n  # ...\n  {MyApp.Endpoint, phoenix_sync: Phoenix.Sync.plug_opts()}\n]\n```\n\nThis is less efficient than running in `:embedded` mode but may be useful for testing or when needing to run an HTTP proxy in front of Electric as part of your development stack.","ref":"readme.html#local-http-services"},{"type":"extras","title":"Different modes for different envs - README","doc":"Apps using `:http` mode in certain environments can exclude `:electric` as a dependency for that environment. The following example shows how to configure:\n\n- `:embedded` mode in `:dev`\n- `:http` mode with a local Electric service in `:test`\n- `:http` mode with an external Electric service in `:prod`\n\nWith Electric only included and compiled as a dependency in `:dev` and `:test`.\n\n```elixir\n# mix.exs\ndefp deps do\n  [\n    {:electric, \"~> 1.0.0-beta.20\", only: [:dev, :test]},\n    {:phoenix_sync, \"~> 0.3\"}\n  ]\nend\n\n# config/dev.exs\nconfig :phoenix_sync,\n  mode: :embedded,\n  env: config_env(),\n  repo: MyApp.Repo\n\n# config/test.esx\nconfig :phoenix_sync,\n  mode: :http,\n  env: config_env(),\n  http: [\n    port: 3000,\n  ],\n  repo: MyApp.Repo,\n  url: \"http://localhost:3000\"\n\n# config/prod.exs\nconfig :phoenix_sync,\n  mode: :http,\n  url: \"https://api.electric-sql.cloud\",\n  credentials: [\n    secret: \"...\",    # required\n    source_id: \"...\"  # optional, required for Electric Cloud\n  ]\n\n# application.ex\nchildren = [\n  MyApp.Repo,\n  # ...\n  {MyApp.Endpoint, phoenix_sync: Phoenix.Sync.plug_opts()}\n]\n```","ref":"readme.html#different-modes-for-different-envs"},{"type":"extras","title":"Notes - README","doc":"","ref":"readme.html#notes"},{"type":"extras","title":"ElectricSQL - README","doc":"ElectricSQL is a [real-time sync engine for Postgres](https://electric-sql.com).\n\nPhoenix.Sync uses Electric to handle the core concerns of partial replication, fan out and data delivery.","ref":"readme.html#electricsql"},{"type":"extras","title":"Partial replication - README","doc":"Electric defines partial replication using [Shapes](https://electric-sql.com/docs/guides/shapes).\n\nPhoenix.Sync maps Ecto queries to shape definitions. This allows you to control what data syncs where using Ecto.Schema and Ecto.Query.","ref":"readme.html#partial-replication"},{"type":"extras","title":"LICENSE","doc":"Apache License\n                           Version 2.0, January 2004\n                        http://www.apache.org/licenses/\n\n   TERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n   1. Definitions.\n\n      \"License\" shall mean the terms and conditions for use, reproduction,\n      and distribution as defined by Sections 1 through 9 of this document.\n\n      \"Licensor\" shall mean the copyright owner or entity authorized by\n      the copyright owner that is granting the License.\n\n      \"Legal Entity\" shall mean the union of the acting entity and all\n      other entities that control, are controlled by, or are under common\n      control with that entity. For the purposes of this definition,\n      \"control\" means (i) the power, direct or indirect, to cause the\n      direction or management of such entity, whether by contract or\n      otherwise, or (ii) ownership of fifty percent (50%) or more of the\n      outstanding shares, or (iii) beneficial ownership of such entity.\n\n      \"You\" (or \"Your\") shall mean an individual or Legal Entity\n      exercising permissions granted by this License.\n\n      \"Source\" form shall mean the preferred form for making modifications,\n      including but not limited to software source code, documentation\n      source, and configuration files.\n\n      \"Object\" form shall mean any form resulting from mechanical\n      transformation or translation of a Source form, including but\n      not limited to compiled object code, generated documentation,\n      and conversions to other media types.\n\n      \"Work\" shall mean the work of authorship, whether in Source or\n      Object form, made available under the License, as indicated by a\n      copyright notice that is included in or attached to the work\n      (an example is provided in the Appendix below).\n\n      \"Derivative Works\" shall mean any work, whether in Source or Object\n      form, that is based on (or derived from) the Work and for which the\n      editorial revisions, annotations, elaborations, or other modifications\n      represent, as a whole, an original work of authorship. For the purposes\n      of this License, Derivative Works shall not include works that remain\n      separable from, or merely link (or bind by name) to the interfaces of,\n      the Work and Derivative Works thereof.\n\n      \"Contribution\" shall mean any work of authorship, including\n      the original version of the Work and any modifications or additions\n      to that Work or Derivative Works thereof, that is intentionally\n      submitted to Licensor for inclusion in the Work by the copyright owner\n      or by an individual or Legal Entity authorized to submit on behalf of\n      the copyright owner. For the purposes of this definition, \"submitted\"\n      means any form of electronic, verbal, or written communication sent\n      to the Licensor or its representatives, including but not limited to\n      communication on electronic mailing lists, source code control systems,\n      and issue tracking systems that are managed by, or on behalf of, the\n      Licensor for the purpose of discussing and improving the Work, but\n      excluding communication that is conspicuously marked or otherwise\n      designated in writing by the copyright owner as \"Not a Contribution.\"\n\n      \"Contributor\" shall mean Licensor and any individual or Legal Entity\n      on behalf of whom a Contribution has been received by Licensor and\n      subsequently incorporated within the Work.\n\n   2. Grant of Copyright License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      copyright license to reproduce, prepare Derivative Works of,\n      publicly display, publicly perform, sublicense, and distribute the\n      Work and such Derivative Works in Source or Object form.\n\n   3. Grant of Patent License. Subject to the terms and conditions of\n      this License, each Contributor hereby grants to You a perpetual,\n      worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n      (except as stated in this section) patent license to make, have made,\n      use, offer to sell, sell, import, and otherwise transfer the Work,\n      where such license applies only to those patent claims licensable\n      by such Contributor that are necessarily infringed by their\n      Contribution(s) alone or by combination of their Contribution(s)\n      with the Work to which such Contribution(s) was submitted. If You\n      institute patent litigation against any entity (including a\n      cross-claim or counterclaim in a lawsuit) alleging that the Work\n      or a Contribution incorporated within the Work constitutes direct\n      or contributory patent infringement, then any patent licenses\n      granted to You under this License for that Work shall terminate\n      as of the date such litigation is filed.\n\n   4. Redistribution. You may reproduce and distribute copies of the\n      Work or Derivative Works thereof in any medium, with or without\n      modifications, and in Source or Object form, provided that You\n      meet the following conditions:\n\n      (a) You must give any other recipients of the Work or\n          Derivative Works a copy of this License; and\n\n      (b) You must cause any modified files to carry prominent notices\n          stating that You changed the files; and\n\n      (c) You must retain, in the Source form of any Derivative Works\n          that You distribute, all copyright, patent, trademark, and\n          attribution notices from the Source form of the Work,\n          excluding those notices that do not pertain to any part of\n          the Derivative Works; and\n\n      (d) If the Work includes a \"NOTICE\" text file as part of its\n          distribution, then any Derivative Works that You distribute must\n          include a readable copy of the attribution notices contained\n          within such NOTICE file, excluding those notices that do not\n          pertain to any part of the Derivative Works, in at least one\n          of the following places: within a NOTICE text file distributed\n          as part of the Derivative Works; within the Source form or\n          documentation, if provided along with the Derivative Works; or,\n          within a display generated by the Derivative Works, if and\n          wherever such third-party notices normally appear. The contents\n          of the NOTICE file are for informational purposes only and\n          do not modify the License. You may add Your own attribution\n          notices within Derivative Works that You distribute, alongside\n          or as an addendum to the NOTICE text from the Work, provided\n          that such additional attribution notices cannot be construed\n          as modifying the License.\n\n      You may add Your own copyright statement to Your modifications and\n      may provide additional or different license terms and conditions\n      for use, reproduction, or distribution of Your modifications, or\n      for any such Derivative Works as a whole, provided Your use,\n      reproduction, and distribution of the Work otherwise complies with\n      the conditions stated in this License.\n\n   5. Submission of Contributions. Unless You explicitly state otherwise,\n      any Contribution intentionally submitted for inclusion in the Work\n      by You to the Licensor shall be under the terms and conditions of\n      this License, without any additional terms or conditions.\n      Notwithstanding the above, nothing herein shall supersede or modify\n      the terms of any separate license agreement you may have executed\n      with Licensor regarding such Contributions.\n\n   6. Trademarks. This License does not grant permission to use the trade\n      names, trademarks, service marks, or product names of the Licensor,\n      except as required for reasonable and customary use in describing the\n      origin of the Work and reproducing the content of the NOTICE file.\n\n   7. Disclaimer of Warranty. Unless required by applicable law or\n      agreed to in writing, Licensor provides the Work (and each\n      Contributor provides its Contributions) on an \"AS IS\" BASIS,\n      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n      implied, including, without limitation, any warranties or conditions\n      of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n      PARTICULAR PURPOSE. You are solely responsible for determining the\n      appropriateness of using or redistributing the Work and assume any\n      risks associated with Your exercise of permissions under this License.\n\n   8. Limitation of Liability. In no event and under no legal theory,\n      whether in tort (including negligence), contract, or otherwise,\n      unless required by applicable law (such as deliberate and grossly\n      negligent acts) or agreed to in writing, shall any Contributor be\n      liable to You for damages, including any direct, indirect, special,\n      incidental, or consequential damages of any character arising as a\n      result of this License or out of the use or inability to use the\n      Work (including but not limited to damages for loss of goodwill,\n      work stoppage, computer failure or malfunction, or any and all\n      other commercial damages or losses), even if such Contributor\n      has been advised of the possibility of such damages.\n\n   9. Accepting Warranty or Additional Liability. While redistributing\n      the Work or Derivative Works thereof, You may choose to offer,\n      and charge a fee for, acceptance of support, warranty, indemnity,\n      or other liability obligations and/or rights consistent with this\n      License. However, in accepting such obligations, You may act only\n      on Your own behalf and on Your sole responsibility, not on behalf\n      of any other Contributor, and only if You agree to indemnify,\n      defend, and hold each Contributor harmless for any liability\n      incurred by, or claims asserted against, such Contributor by reason\n      of your accepting any such warranty or additional liability.\n\n   END OF TERMS AND CONDITIONS","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.1"}}